@@ -302,6 +302,7 @@ Parser.prototype._parseSubscribe = function () {
     // Parse topic
     topic = this._parseString()
     if (topic === null) return this._emitError(new Error('Cannot parse topic'))
+    if (this._pos >= packet.length) return this._emitError(new Error('Malformed Subscribe Payload'))
 
     options = this._parseByte()
     qos = options & constants.SUBSCRIBE_OPTIONS_QOS_MASK
@@ -597,10 +598,29 @@ Parser.prototype._parseProperties = function () {
         result[name] = {}
       }
       var currentUserProperty = this._parseByType(constants.propertiesTypes[name])
-      result[name][currentUserProperty.name] = currentUserProperty.value
+      if (result[name][currentUserProperty.name]) {
+        if (Array.isArray(result[name][currentUserProperty.name])) {
+          result[name][currentUserProperty.name].push(currentUserProperty.value)
+        } else {
+          var currentValue = result[name][currentUserProperty.name]
+          result[name][currentUserProperty.name] = [currentValue]
+          result[name][currentUserProperty.name].push(currentUserProperty.value)
+        }
+      } else {
+        result[name][currentUserProperty.name] = currentUserProperty.value
+      }
       continue
     }
-    result[name] = this._parseByType(constants.propertiesTypes[name])
+    if (result[name]) {
+      if (Array.isArray(result[name])) {
+        result[name].push(this._parseByType(constants.propertiesTypes[name]))
+      } else {
+        result[name] = [result[name]]
+        result[name].push(this._parseByType(constants.propertiesTypes[name]))
+      }
+    } else {
+      result[name] = this._parseByType(constants.propertiesTypes[name])
+    }
   }
   return result
 }
