@@ -260,6 +260,79 @@ testParseGenerate('connect MQTT 5.0', {
   4, 3, 2, 1// Will payload
 ]))
 
+testParseGenerate('connect MQTT 5.0 with will properties but w/o will payload', {
+  cmd: 'connect',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 121,
+  protocolId: 'MQTT',
+  protocolVersion: 5,
+  will: {
+    retain: true,
+    qos: 2,
+    properties: {
+      willDelayInterval: 1234,
+      payloadFormatIndicator: false,
+      messageExpiryInterval: 4321,
+      contentType: 'test',
+      responseTopic: 'topic',
+      correlationData: Buffer.from([1, 2, 3, 4]),
+      userProperties: {
+        'test': 'test'
+      }
+    },
+    topic: 'topic',
+    payload: Buffer.from([])
+  },
+  clean: true,
+  keepalive: 30,
+  properties: {
+    sessionExpiryInterval: 1234,
+    receiveMaximum: 432,
+    maximumPacketSize: 100,
+    topicAliasMaximum: 456,
+    requestResponseInformation: true,
+    requestProblemInformation: true,
+    userProperties: {
+      'test': 'test'
+    },
+    authenticationMethod: 'test',
+    authenticationData: Buffer.from([1, 2, 3, 4])
+  },
+  clientId: 'test'
+}, Buffer.from([
+  16, 121, // Header
+  0, 4, // Protocol ID length
+  77, 81, 84, 84, // Protocol ID
+  5, // Protocol version
+  54, // Connect flags
+  0, 30, // Keepalive
+  47, // properties length
+  17, 0, 0, 4, 210, // sessionExpiryInterval
+  33, 1, 176, // receiveMaximum
+  39, 0, 0, 0, 100, // maximumPacketSize
+  34, 1, 200,  // topicAliasMaximum
+  25, 1, // requestResponseInformation
+  23, 1, // requestProblemInformation,
+  38, 0, 4, 116, 101, 115, 116, 0, 4, 116, 101, 115, 116, // userProperties,
+  21, 0, 4, 116, 101, 115, 116, // authenticationMethod
+  22, 0, 4, 1, 2, 3, 4, // authenticationData
+  0, 4, // Client ID length
+  116, 101, 115, 116, // Client ID
+  47, // will properties
+  24, 0, 0, 4, 210, // will delay interval
+  1, 0, // payload format indicator
+  2, 0, 0, 16, 225, // message expiry interval
+  3, 0, 4, 116, 101, 115, 116, // content type
+  8, 0, 5, 116, 111, 112, 105, 99, // response topic
+  9, 0, 4, 1, 2, 3, 4, // corelation data
+  38, 0, 4, 116, 101, 115, 116, 0, 4, 116, 101, 115, 116, // user properties
+  0, 5, // Will topic length
+  116, 111, 112, 105, 99, // Will topic
+  0, 0 // Will payload length
+]))
+
 testParseGenerate('connect MQTT 5.0 w/o will properties', {
   cmd: 'connect',
   retain: false,
@@ -661,7 +734,7 @@ testParseGenerate('connack MQTT5 with properties', {
   retain: false,
   qos: 0,
   dup: false,
-  length: 89,
+  length: 87,
   sessionPresent: false,
   reasonCode: 0,
   properties: {
@@ -686,8 +759,8 @@ testParseGenerate('connack MQTT5 with properties', {
     authenticationData: Buffer.from([1, 2, 3, 4])
   }
 }, Buffer.from([
-  32, 89, 0, 0,
-  86, // properties length
+  32, 87, 0, 0,
+  84, // properties length
   17, 0, 0, 4, 210, // sessionExpiryInterval
   33, 1, 176, // receiveMaximum
   36, 2, // Maximum qos
@@ -700,7 +773,59 @@ testParseGenerate('connack MQTT5 with properties', {
   40, 1, // wildcardSubscriptionAvailable
   41, 1, // subscriptionIdentifiersAvailable
   42, 0, // sharedSubscriptionAvailable
-  19, 0, 0, 4, 210, // serverKeepAlive
+  19, 4, 210, // serverKeepAlive
+  26, 0, 4, 116, 101, 115, 116, // responseInformation
+  28, 0, 4, 116, 101, 115, 116, // serverReference
+  21, 0, 4, 116, 101, 115, 116, // authenticationMethod
+  22, 0, 4, 1, 2, 3, 4 // authenticationData
+]), { protocolVersion: 5 })
+
+testParseGenerate('connack MQTT5 with properties and doubled user properties', {
+  cmd: 'connack',
+  retain: false,
+  qos: 0,
+  dup: false,
+  length: 100,
+  sessionPresent: false,
+  reasonCode: 0,
+  properties: {
+    sessionExpiryInterval: 1234,
+    receiveMaximum: 432,
+    maximumQoS: 2,
+    retainAvailable: true,
+    maximumPacketSize: 100,
+    assignedClientIdentifier: 'test',
+    topicAliasMaximum: 456,
+    reasonString: 'test',
+    userProperties: {
+      'test': ['test', 'test']
+    },
+    wildcardSubscriptionAvailable: true,
+    subscriptionIdentifiersAvailable: true,
+    sharedSubscriptionAvailable: false,
+    serverKeepAlive: 1234,
+    responseInformation: 'test',
+    serverReference: 'test',
+    authenticationMethod: 'test',
+    authenticationData: Buffer.from([1, 2, 3, 4])
+  }
+}, Buffer.from([
+  32, 100, 0, 0,
+  97, // properties length
+  17, 0, 0, 4, 210, // sessionExpiryInterval
+  33, 1, 176, // receiveMaximum
+  36, 2, // Maximum qos
+  37, 1, // retainAvailable
+  39, 0, 0, 0, 100, // maximumPacketSize
+  18, 0, 4, 116, 101, 115, 116, // assignedClientIdentifier
+  34, 1, 200, // topicAliasMaximum
+  31, 0, 4, 116, 101, 115, 116, // reasonString
+  38, 0, 4, 116, 101, 115, 116, 0, 4, 116, 101, 115, 116,
+  38, 0, 4, 116, 101, 115, 116, 0, 4, 116, 101, 115, 116, // userProperties
+  40, 1, // wildcardSubscriptionAvailable
+  41, 1, // subscriptionIdentifiersAvailable
+  42, 0, // sharedSubscriptionAvailable
+  19, 4, 210, // serverKeepAlive
   26, 0, 4, 116, 101, 115, 116, // responseInformation
   28, 0, 4, 116, 101, 115, 116, // serverReference
   21, 0, 4, 116, 101, 115, 116, // authenticationMethod
@@ -784,6 +909,45 @@ testParseGenerate('publish MQTT5 properties', {
   116, 101, 115, 116 // Payload (test)
 ]), { protocolVersion: 5 })
 
+testParseGenerate('publish MQTT5 with multiple same properties', {
+  cmd: 'publish',
+  retain: true,
+  qos: 2,
+  dup: true,
+  length: 62,
+  topic: 'test',
+  payload: new Buffer('test'),
+  messageId: 10,
+  properties: {
+    payloadFormatIndicator: true,
+    messageExpiryInterval: 4321,
+    topicAlias: 100,
+    responseTopic: 'topic',
+    correlationData: Buffer.from([1, 2, 3, 4]),
+    userProperties: {
+      'test': 'test'
+    },
+    subscriptionIdentifier: [120, 121],
+    contentType: 'test'
+  }
+}, Buffer.from([
+  61, 62, // Header
+  0, 4, // Topic length
+  116, 101, 115, 116, // Topic (test)
+  0, 10, // Message ID
+  49, // properties length
+  1, 1, // payloadFormatIndicator
+  2, 0, 0, 16, 225, // message expiry interval
+  35, 0, 100, // topicAlias
+  8, 0, 5, 116, 111, 112, 105, 99, // response topic
+  9, 0, 4, 1, 2, 3, 4, // correlationData
+  38, 0, 4, 116, 101, 115, 116, 0, 4, 116, 101, 115, 116, // userProperties
+  11, 120, // subscriptionIdentifier
+  11, 121, // subscriptionIdentifier
+  3, 0, 4, 116, 101, 115, 116, // content type
+  116, 101, 115, 116 // Payload (test)
+]), { protocolVersion: 5 })
+
 ;(function () {
   var buffer = new Buffer(2048)
   testParseGenerate('2KB publish packet', {
@@ -1714,6 +1878,17 @@ testParseError('Not supported auth packet for this version MQTT', Buffer.from([
   38, 0, 4, 116, 101, 115, 116, 0, 4, 116, 101, 115, 116 // userProperties
 ]))
 
+// When a Subscribe packet contains a topic_filter and the given
+// length is topic_filter.length + 1 then the last byte (requested QoS) is interpreted as topic_filter
+// reading the requested_qos at the end causes 'Index out of range' read
+testParseError('Malformed Subscribe Payload', Buffer.from([
+  130, 14, // subscribe header and remaining length
+  0, 123,  // packet ID
+  0, 10,   // topic filter length
+  104, 105, 106, 107, 108, 47, 109, 110, 111,  // topic filter with length of 9 bytes
+  0       // requested QoS
+]))
+
 test('stops parsing after first error', function (t) {
   t.plan(4)
 
